/* Marmote is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Marmote is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Marmote. If not, see <http://www.gnu.org/licenses/>.

Copyright 2015 Alain Jean-Marie, Jean-Michel Fourneau, Jean-Marc Vincent, Issam Rabhi */

#ifndef HOMOGENOUS1DRANDOMWALK_H
#define HOMOGENOUS1DRANDOMWALK_H

#include "markovChain.h"
#include "Distribution/discreteDistribution.h"
#include "Distribution/geometricDistribution.h"

/**
 * @brief The 1-dimensional random walk with homogeneous transition probabilities.
 * This model is characterized by:
 * - the size N, possibly INFINITE_STATE_SPACE_SIZE: number of states
 * - the probability to jump to the right, p
 * - the probability to jump to the left, q
 * with p + q <= 1. The probability to stay at the same position is 1 - p - q.
*/

class homogeneous1DRandomWalk : public markovChain 
{
 protected:
  int _size;		/**< @brief the number of states; possibly infinite */
  double* _stateSpace;	/**< @brief representation of the state space when finite */
  double _p;            /**< @brief the probability to jump to the right */
  double _q;            /**< @brief the probability to jump to the left */
  double _r; 		/**< @brief r = 1 - p - q is the proba of staying in the same state */

 public:
  homogeneous1DRandomWalk( double l, double m );
  homogeneous1DRandomWalk( int sz, double l, double m );
  ~homogeneous1DRandomWalk();

  void makeMarkovChain();

  discreteDistribution* transientDistribution( int t, int nMax );
  geometricDistribution* stationaryDistribution();
  discreteDistribution* stationaryDistribution( int nMax );
  simulationResult*
    simulateChain( long int tMax, bool stat, bool traj, bool trace );

 public:
  void write(string format, string prefix);

};

#endif // HOMOGENOUS1DRANDOMWALK_H
